=== modified file 'lib/canonical/librarian/client.py'
--- lib/canonical/librarian/client.py	2011-06-01 15:01:45 +0000
+++ lib/canonical/librarian/client.py	2011-09-03 03:51:05 +0000
@@ -34,6 +34,7 @@
 from zope.interface import implements
 
 from canonical.config import config, dbconfig
+from canonical.database.postgresql import ConnectionString
 from canonical.launchpad.webapp.interfaces import (
         IStoreSelector, MAIN_STORE, MASTER_FLAVOR)
 from canonical.librarian.interfaces import (
@@ -226,9 +227,10 @@
         try:
             # Use dbconfig.rw_main_master directly here because it doesn't
             # make sense to try and use ro_main_master (which might be
-            # returned if we use dbconfig.main_master).
-            database_name = re.search(
-                r"dbname=(\S*)", dbconfig.rw_main_master).group(1)
+            # returned if we use dbconfig.main_master). Note we can't
+            # use database introspection, as not all clients using this
+            # method have database access.
+            database_name = ConnectionString(dbconfig.rw_main_master).dbname
             self._sendLine('STORE %d %s' % (size, name))
             self._sendHeader('Database-Name', database_name)
             self._sendHeader('Content-Type', str(contentType))

=== modified file 'lib/canonical/librarian/storage.py'
--- lib/canonical/librarian/storage.py	2010-09-24 15:40:49 +0000
+++ lib/canonical/librarian/storage.py	2011-09-03 03:40:27 +0000
@@ -11,6 +11,8 @@
 
 from zope.component import getUtility
 
+from canonical.config import dbconfig
+from canonical.database.postgresql import ConnectionString
 from canonical.launchpad.webapp.interfaces import (
         IStoreSelector, MAIN_STORE, DEFAULT_FLAVOR)
 from lp.services.database import write_transaction
@@ -123,12 +125,21 @@
             # If the client told us the name of the database it's using,
             # check that it matches.
             if self.databaseName is not None:
+                # Per Bug #840068, there are two methods of getting the
+                # database name (connection string and db
+                # introspection), and they can give different results
+                # due to pgbouncer database aliases. Lets check both,
+                # and succeed if either matches.
+                config_dbname = ConnectionString(
+                    dbconfig.rw_main_master).dbname
+
                 store = getUtility(IStoreSelector).get(
                         MAIN_STORE, DEFAULT_FLAVOR)
                 result = store.execute("SELECT current_database()")
-                databaseName = result.get_one()[0]
-                if self.databaseName != databaseName:
-                    raise WrongDatabaseError(self.databaseName, databaseName)
+                real_dbname = result.get_one()[0]
+                if self.databaseName not in (config_dbname, real_dbname):
+                    raise WrongDatabaseError(
+                        self.databaseName, (config_dbname, real_dbname))
 
             self.debugLog.append(
                 'database name %r ok' % (self.databaseName, ))

